// Copyright (c) 2023 Citrix Systems, Inc. All Rights Reserved.
// You may only reproduce, distribute, perform, display, or prepare derivative works of this file pursuant to a valid license from Citrix.
// THIS SAMPLE CODE IS PROVIDED BY CITRIX "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

#pragma once

#include <string>
#include <vector>
#include <stdint.h>
#include <curl/curl.h>

#include "CitrixAuthManagerCustomProtocolSDK.h"

// A simple class to help make the single HTTP POST request to StoreFront.
// The Token Request is passed as the body of the POST and the certificate
// is passed as a base 64 encoded custom header.
class PostCertificateAndTokenRequest
{
public:
    PostCertificateAndTokenRequest();
    ~PostCertificateAndTokenRequest();

    // Sends the token request and certificate to StoreFront and returns
    // the token response data and eAuthResult code.
    CitrixAuthManagerCustomProtocolSDK::eAuthResult PostRequestAndCertificateToAuthExtension(const std::string& url,
                                                                                             const std::string& tokenRequest,
                                                                                             const std::vector<char>& certificateFileContent,
                                                                                             /* out */ std::vector<char>& outServerResponse);

private:
    // Helper to allow easy clean up of the curl handles.
    CitrixAuthManagerCustomProtocolSDK::eAuthResult PostRequestAndCertificateToAuthExtensionInternal(const std::string& startURL,
                                                                                                     const std::string& tokenRequest,
                                                                                                     const std::vector<char>& certificateFileContent,
                                                                                                     /* out */ std::vector<char>& outServerResponse);

    void CloseHandles();
    size_t writeHeaderHandleImpl(char* buffer, size_t size, size_t nmemb);
    static size_t writeHeaderHandle(char* buffer, size_t size, size_t nmemb, void* userp);
    size_t writeResponseHandleImpl(char* buffer, size_t size, size_t nmemb);
    static size_t writeResponseHandle(char* buffer, size_t size, size_t nmemb, void* userp);

private:
    // The curl handle to make clean up easy.
    CURL *m_curlEasyHandle;

    // The list of headers for the request.
    struct curl_slist* m_headers;

    // The response.
    std::vector<std::string> m_responseHeaders;
    std::string m_responseData;
};
